import json

import wagtail
from django.core.exceptions import PermissionDenied
from django.shortcuts import redirect, render
from django.utils.http import url_has_allowed_host_and_scheme
from django.utils.translation import gettext as _
from treebeard.mp_tree import MP_MoveHandler
from wagtail.admin import messages
from wagtail.core.models import Page

from .forms import MoveForm
from .models import TrashCan
from .utils import generate_page_data, restore_and_move_page, trash_can_for_request


def get_valid_next_url_from_request(request):
    next_url = request.POST.get("next") or request.GET.get("next")
    if not next_url or not url_has_allowed_host_and_scheme(
        url=next_url, allowed_hosts={request.get_host()}
    ):
        return ""
    return next_url


def trash_delete(request, page):
    if not request.method == "POST":
        return

    trash_can = trash_can_for_request(request)

    parent = page.get_parent()

    if parent.id == trash_can.id:
        page.delete(user=request.user)

        messages.success(
            request, _("Page '{0}' deleted.").format(page.get_admin_display_title())
        )
    else:
        TrashCan.objects.create(
            page=page, parent=parent, user=request.user, data=generate_page_data(page)
        )

        page.get_descendants(inclusive=True).unpublish()

        if wagtail.VERSION >= (2, 16):
            # Preserve the url path
            old_page = Page.objects.get(id=page.id)
            new_url_path = old_page.set_url_path(parent=trash_can)

            MP_MoveHandler(page, trash_can, "first-child").process()

            # And reset the url path when in trash
            new_page = Page.objects.get(id=page.id)
            new_page.url_path = new_url_path
            new_page.save()
        else:
            page.move(trash_can, pos="first-child", user=request.user)

        messages.success(
            request,
            _("Page '{0}' moved to trash_can.").format(page.get_admin_display_title()),
        )

    next_url = get_valid_next_url_from_request(request)

    if next_url:
        return redirect(next_url)

    return redirect("wagtailadmin_explore", parent.id)


def trash_move(request, page_id):
    if request.method == "POST":
        rb = TrashCan.objects.get(page_id=page_id)
        move_to_page = Page.objects.get(pk=request.POST.get("move_page"))
        restore_and_move_page(rb, move_to_page, request)

        messages.success(
            request,
            _("Page '{0}' successfully restored.").format(
                rb.page.get_admin_display_title()
            ),
        )

        return redirect("wagtailadmin_explore", rb.page_id)

    return render(
        request,
        "wagtail_trash/move.html",
        {
            "form": MoveForm(),
        },
    )


def trash_restore(request, page_id):
    rb = TrashCan.objects.get(page_id=page_id)
    page = rb.page

    if not page.permissions_for_user(request.user).can_edit():
        raise PermissionDenied

    restore_and_move_page(rb, rb.parent, request)

    messages.success(
        request,
        _("Page '{0}' successfully restored.").format(page.get_admin_display_title()),
    )

    return redirect("wagtailadmin_explore", page_id)



# Generated by Django 3.1.7 on 2021-11-16 13:50

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('wagtailcore', '0062_comment_models_and_pagesubscription'),
        ('wagtail_trash', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='trashcan',
            options={'verbose_name': 'Trash Can', 'verbose_name_plural': 'Trash Cans'},
        ),
        migrations.AlterField(
            model_name='trashcan',
            name='data',
            field=models.TextField(blank=True, verbose_name='Data'),
        ),
        migrations.AlterField(
            model_name='trashcan',
            name='page',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='wagtailcore.page', verbose_name='Page'),
        ),
        migrations.AlterField(
            model_name='trashcan',
            name='parent',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailcore.page', verbose_name='Parent'),
        ),
        migrations.AlterField(
            model_name='trashcan',
            name='time_recycled',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Time Recycled'),
        ),
        migrations.AlterField(
            model_name='trashcan',
            name='user',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
    ]


from django.contrib.auth import get_user_model
from django.db import models
from django.utils.translation import gettext_lazy as _
from wagtail.core.models import Page


class TrashCan(models.Model):
    page = models.ForeignKey(
        Page, on_delete=models.CASCADE, related_name="+", verbose_name=_("Page")
    )
    parent = models.ForeignKey(
        Page,
        null=True,
        on_delete=models.SET_NULL,
        related_name="+",
        verbose_name=_("Parent"),
    )
    user = models.ForeignKey(
        get_user_model(), null=True, on_delete=models.SET_NULL, verbose_name=_("User")
    )
    time_recycled = models.DateTimeField(_("Time Recycled"), auto_now_add=True)
    data = models.TextField(_("Data"), blank=True)

    def __str__(self):
        return self.page.title

    class Meta:
        verbose_name = _("Trash Can")
        verbose_name_plural = _("Trash Cans")


class TrashCanPage(Page):
    parent_page_types = []
    subpage_types = []



from django.shortcuts import reverse
from django.urls import path
from django.utils.safestring import mark_safe
from django.utils.translation import gettext as _
from wagtail.contrib.modeladmin.helpers import ButtonHelper, PermissionHelper
from wagtail.contrib.modeladmin.options import ModelAdmin, modeladmin_register
from wagtail.contrib.modeladmin.views import IndexView
from wagtail.core import hooks
from wagtail.core.models import Page

from .models import TrashCan, TrashCanPage
from .utils import trash_can_for_request
from .views import trash_delete, trash_move, trash_restore


class TrashPermissionHelper(PermissionHelper):
    def user_can_create(self, user):
        return False


class TrashButtonHelper(ButtonHelper):
    restore_button_classnames = [
        "button-small",
        "button-secondary",
        "icon",
        "icon-undo",
    ]

    def restore_and_move_button(self, obj):
        return {
            "url": reverse("wagtail_trash_move", args=(obj.page.id,)),
            "label": _("Restore and move"),
            "classname": self.finalise_classname(self.restore_button_classnames),
            "title": _("Restore and move"),
        }

    def restore_button(self, obj):
        return {
            "url": reverse("wagtail_trash_restore", args=(obj.page.id,)),
            "label": _("Restore"),
            "classname": self.finalise_classname(self.restore_button_classnames),
            "title": _("Restore"),
        }

    def has_ancestor_in_bin(self, obj):
        parent = obj.get_parent()

        if not parent:
            return False

        ancestor_app_labels = list(
            parent.get_ancestors(inclusive=True).values_list(
                "content_type__app_label", flat=True
            )
        )

        return "wagtail_trash" in ancestor_app_labels

    def get_buttons_for_obj(
        self, obj, exclude=["edit"], classnames_add=None, classnames_exclude=None
    ):
        buttons = super().get_buttons_for_obj(
            obj, exclude, classnames_add, classnames_exclude
        )

        if "restore" not in (exclude or []):
            parent = obj.parent
            if parent and not self.has_ancestor_in_bin(parent):
                buttons.append(self.restore_button(obj))
            buttons.append(self.restore_and_move_button(obj))

        return buttons


class TrashCanIndexView(IndexView):
    def get_page_title(self):
        return _("Trash Can")


class TrashCanModelAdmin(ModelAdmin):
    model = TrashCan
    menu_label = _("Trash Can")
    menu_icon = "bin"
    admin_order_field = "title"

    list_display = ("page_tree", "time_recycled", "user")
    # search_fields = ("title",)

    button_helper_class = TrashButtonHelper
    permission_helper_class = TrashPermissionHelper

    index_view_class = TrashCanIndexView
    index_template_name = "wagtail_trash/index.html"

    def page_tree(self, rb):
        descendants = rb.page.get_descendants(inclusive=True)

        depth = rb.page.depth
        output = ""

        for i, x in enumerate(descendants):
            output += "&nbsp;" * (x.depth - depth)

            if i == 0:
                output += "<strong>" + x.title + "</strong><br>"
            else:
                output += x.title + "<br>"

        return mark_safe(output)

    def get_queryset(self, request):
        # Create a bin if there is none
        trash_can_for_request(request)

        return super().get_queryset(request).prefetch_related("page")


modeladmin_register(TrashCanModelAdmin)


@hooks.register("before_delete_page")
def delete_page(request, page):
    return trash_delete(request, page)


@hooks.register("construct_page_chooser_queryset")
def exclude_trash_can_from_chooser(pages, request):
    pages = pages.not_type(TrashCanPage)

    return pages


@hooks.register("construct_explorer_page_queryset")
def exclude_trash_can_from_explorer(parent_page, pages, request):
    pages = pages.not_type(TrashCanPage)

    return pages


@hooks.register("register_admin_urls")
def urlconf_time():
    return [
        path(
            "wagtail_trash/restore_and_move/<int:page_id>/",
            trash_move,
            name="wagtail_trash_move",
        ),
        path(
            "wagtail_trash/restore/<int:page_id>/",
            trash_restore,
            name="wagtail_trash_restore",
        ),
    ]



from datetime import timedelta

from django.core.management.base import BaseCommand
from django.utils import timezone

from wagtail_trash.models import TrashCan


class Command(BaseCommand):
    """
    Remove pages from recycle bin

    Example:
        ./manage.py empty_trash --older_than_days=30

    Will remove all pages that were deleted more than 30 days ago
    """

    def add_arguments(self, parser):
        parser.add_argument("--older_than_days", required=True)

    def handle(self, *args, **options):
        days = int(options["older_than_days"])

        removal_date = timezone.now() - timedelta(days=days)

        rbs = TrashCan.objects.filter(time_recycled__lt=removal_date)

        for rb in rbs:
            rb.page.delete()



import wagtail
from django.db import models

if wagtail.VERSION >= (3, 0):
    from wagtail.query import PageQuerySet
else:
    from wagtail.core.query import PageQuerySet

from wagtail_trash.models import TrashCanPage


class TrashCanPageQuerySet(PageQuerySet):
    def exclude_trash(self):
        bins = TrashCanPage.objects.all()

        if not bins.exists():
            return self

        bin_paths = bins.values_list("path", flat=True)

        q = models.Q()

        for path in bin_paths:
            q |= models.Q(path__startswith=path)

        return self.exclude(q)


class BaseTrashManager(models.Manager):
    def get_queryset(self):
        return (super().get_queryset()).exclude_trash()


TrashManager = BaseTrashManager.from_queryset(TrashCanPageQuerySet)



from django import forms
from django.utils.translation import gettext as _
from wagtail.admin.widgets import AdminPageChooser


class MoveForm(forms.Form):
    move_page = forms.CharField(
        widget=AdminPageChooser(
            choose_one_text=_("Choose a root page"),
            choose_another_text=_("Choose a different root page"),
        )
    )



# Generated by Django 3.1.2 on 2020-10-29 13:38

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0052_pagelogentry'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='TrashCanPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.page')),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
        migrations.CreateModel(
            name='TrashCan',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('time_recycled', models.DateTimeField(auto_now_add=True)),
                ('data', models.TextField(blank=True)),
                ('page', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='wagtailcore.page')),
                ('parent', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailcore.page')),
                ('user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



#!/usr/bin/env python
from setuptools import find_packages, setup

with open("wagtail_trash/version.py", "r") as f:
    version = None
    exec(f.read())

with open("README.md", "r") as f:
    readme = f.read()

testing_extras = ["black", "wagtail-factories"]

setup(
    name="wagtail-trash",
    version=version,
    description="Make deleted pages only temporarily deleted.",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="Andreas Bernacca",
    author_email="ante.bernacca@gmail.com",
    install_requires=[
        "wagtail>=2.14",
    ],
    extras_require={
        "testing": testing_extras,
    },
    setup_requires=["wheel"],
    zip_safe=False,
    license="MIT License",
    packages=find_packages(exclude=["tests*"]),
    include_package_data=True,
    package_data={},
    classifiers=[
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Framework :: Django",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.0",
        "Framework :: Wagtail",
        "Framework :: Wagtail :: 2",
        "Framework :: Wagtail :: 3",
        "License :: OSI Approved :: MIT License",
    ],
    project_urls={
        "Source": "https://github.com/Frojd/wagtail-trash/",
        "Changelog": "https://github.com/Frojd/wagtail-trash/blob/main/CHANGELOG.md",
    },
)



import json

import wagtail
from django.core.exceptions import PermissionDenied
from django.utils.translation import gettext as _

if wagtail.VERSION >= (3, 0):
    from wagtail.actions.move_page import MovePageAction
elif wagtail.VERSION >= (2, 16):
    from wagtail.core.actions.move_page import MovePageAction

from wagtail.core.models import Page, Site

from .models import TrashCanPage


def trash_can_for_request(request):
    site = Site.find_for_request(request)
    trash_can = TrashCanPage.objects.in_site(site)

    if not trash_can.exists():
        trash_can = TrashCanPage(
            title=_("Trash Can"),
            has_unpublished_changes=True,
            live=False,
        )
        site.root_page.add_child(instance=trash_can)
        trash_can.save_revision()
    else:
        trash_can = trash_can.first()

    return trash_can


def generate_page_data(page):
    id_list = []

    if page.live:
        id_list.append(page.id)

    id_list.extend(page.get_descendants().live().values_list("id", flat=True))

    data = {
        "published": id_list,
    }

    return json.dumps(data)


def restore_and_move_page(rb, move_to_page, request):
    if not rb.page.permissions_for_user(request.user).can_move():
        raise PermissionDenied

    if wagtail.VERSION >= (2, 16):
        action = MovePageAction(
            rb.page, move_to_page, pos="first-child", user=request.user
        )
        action.execute(skip_permission_checks=True)
    else:
        rb.page.move(move_to_page, pos="first-child", user=request.user)

    to_be_published_ids = json.loads(rb.data)["published"]

    to_publish_pages = Page.objects.filter(id__in=to_be_published_ids)

    for page in to_publish_pages:
        # Should we make a revision here instead?
        page.has_unpublished_changes = False
        page.live = True
        page.save()

    rb.delete()
